package Util::Medley::Roles::Attributes::Linux::PkgMgmt::RPM;
$Util::Medley::Roles::Attributes::Linux::PkgMgmt::RPM::VERSION = '0.054';
use Modern::Perl;
use Moose::Role;
use Util::Medley::Linux::PkgMgmt::RPM;

=head1 NAME

Util::Medley::Roles::Attributes::Linux::PkgMgmt::RPM

=head1 VERSION

version 0.054

=cut

has LinuxPkgMgmtRpm => (
	is      => 'ro',
	isa     => 'Util::Medley::Linux::PkgMgmt::RPM',
	lazy    => 1,
	default => sub { return Util::Medley::Linux::PkgMgmt::RPM->new; }
);

1;
